package main

import (
	"context"
	"fmt"
	"math/rand"
	"strconv"
	"time"

	"github.com/go-telegram/bot"
	"github.com/go-telegram/bot/models"
)

func GetChatFullName(chat models.Chat) string {
	if chat.Title != "" {
		return chat.Title
	}
	if chat.FirstName != "" {
		if chat.LastName != "" {
			return fmt.Sprint(chat.FirstName, " ", chat.LastName)
		}
		return chat.FirstName
	}
	return ""
}

func GetUserFullName(user models.User) string {
	if user.FirstName != "" {
		if user.LastName != "" {
			return fmt.Sprint(user.FirstName, " ", user.LastName)
		}
		return user.FirstName
	}
	return ""
}

func FetchMemberFromChat(ctx context.Context, b *bot.Bot, chatID int64, userID int64) (*models.ChatMember, error) {
	return b.GetChatMember(ctx, &bot.GetChatMemberParams{ChatID: chatID, UserID: userID})
}

func IsMemberAdmin(member models.ChatMember) bool {
	return member.Administrator != nil || member.Owner != nil
}

func DeleteAfterSeconds(chatID int64, msgID int, seconds int) error {
	return AddMessageToDelete(MessageToDelete{MessageId: msgID, ChatId: chatID, DeleteDate: time.Now().Add(time.Duration(seconds) * time.Second).Unix(), Tries: 0})
}

func generateRandomDigits(length int) string {
	digits := make([]byte, length)

	for i := 0; i < length; i++ {
		digits[i] = byte(rand.Intn(10) + '0') // Generate a random digit
	}

	return string(digits)
}

func Mention(name string, id int64) string {
	text := fmt.Sprintf("[%s](tg://user?id=%d)", bot.EscapeMarkdown(name), id)
	return text
}

func pluralRu(n int, single string, double string, five string) string {
	switch n {
	case 10, 11, 12, 13, 14, 15, 16, 17, 18, 19:
		return five
	default:
		s := []rune(strconv.Itoa(n))
		switch s[len(s)-1] {
		case '1':
			return single
		case '2', '3', '4':
			return double
		default:
			return five
		}
	}
}
