package main

import (
	"context"
	"log"
	"strings"

	"github.com/go-telegram/bot"
	"github.com/go-telegram/bot/models"
)

func checkRegistered(next bot.HandlerFunc) bot.HandlerFunc {
	return func(ctx context.Context, b *bot.Bot, update *models.Update) {
		if update.Message == nil {
			next(ctx, b, update)
			return
		}
		chat := update.Message.Chat
		if chat.Type == "private" {
			next(ctx, b, update)
			return
		}
		if !IsChatActive(chat.ID) && !strings.HasPrefix(update.Message.Text, "/register") {
			log.Println("checkRegistered: not registered", chat.ID)
			return
		}

		next(ctx, b, update)
	}
}

func logChats(next bot.HandlerFunc) bot.HandlerFunc {
	return func(ctx context.Context, b *bot.Bot, update *models.Update) {
		if update.Message == nil {
			next(ctx, b, update)
			return
		}
		chat := update.Message.Chat
		chatName := GetChatFullName(chat)
		if !IsChatExists(chat.ID) {
			db.MustExec(`insert into chats (id, username, name, topic, active) values ($1, $2, $3, $4, $5);`, chat.ID, chat.Username, chatName, 0, false)
		} else {
			db.MustExec(`update chats set username = $2, name = $3 where id = $1;`, chat.ID, chat.Username, chatName)
		}
		next(ctx, b, update)
	}
}

func logUsers(next bot.HandlerFunc) bot.HandlerFunc {
	return func(ctx context.Context, b *bot.Bot, update *models.Update) {
		if update.Message == nil {
			next(ctx, b, update)
			return
		}
		user := update.Message.From
		userFullName := GetUserFullName(*user)
		if !IsUserExists(user.ID) {
			db.MustExec(`insert into users (id, username, name) values ($1, $2, $3);`, user.ID, user.Username, userFullName, 0, false)
		} else {
			db.MustExec(`update users set username = $2, name = $3 where id = $1;`, user.ID, user.Username, userFullName)
		}
		next(ctx, b, update)
	}
}
